/*
 * Copyright (C) 2018-2022 Toshiaki Maki <makingx@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.xbib.datastructures.validation.builder;

import java.util.function.Function;

import org.xbib.datastructures.validation.arguments.Arguments1;
import org.xbib.datastructures.validation.arguments.IntegerValidator;
import org.xbib.datastructures.validation.constraint.IntegerConstraint;
import org.xbib.datastructures.validation.core.Validator;

/**
 * @since 0.7.0
 */
public class IntegerValidatorBuilder {
	private final String name;

	private final Function<IntegerConstraint<Arguments1<Integer>>, IntegerConstraint<Arguments1<Integer>>> constraints;

	public static IntegerValidatorBuilder of(String name,
			Function<IntegerConstraint<Arguments1<Integer>>, IntegerConstraint<Arguments1<Integer>>> constraints) {
		return new IntegerValidatorBuilder(name, constraints);
	}

	IntegerValidatorBuilder(String name,
			Function<IntegerConstraint<Arguments1<Integer>>, IntegerConstraint<Arguments1<Integer>>> constraints) {
		this.name = name;
		this.constraints = constraints;
	}

	public <T> IntegerValidator<T> build(Function<? super Integer, ? extends T> mapper) {
		final Validator<Arguments1<Integer>> validator = ValidatorBuilder
				.<Arguments1<Integer>> of()
				.constraint(Arguments1::arg1, name, constraints).build();
		return new IntegerValidator<>(validator, mapper::apply);
	}

	public IntegerValidator<Integer> build() {
		return build(x -> x);
	}
}
