/*
 * Copyright (C) 2018-2022 Toshiaki Maki <makingx@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.xbib.datastructures.validation.constraint;

import java.time.Clock;
import java.time.LocalTime;

import org.xbib.datastructures.validation.constraint.base.TemporalConstraintBase;

/**
 * This is the actual class for constraints on LocalTime.
 */
public class LocalTimeConstraint<T>
		extends TemporalConstraintBase<T, LocalTime, LocalTimeConstraint<T>> {
	@Override
	protected boolean isAfter(LocalTime a, LocalTime b) {
		return a.isAfter(b);
	}

	@Override
	protected boolean isBefore(LocalTime a, LocalTime b) {
		return a.isBefore(b);
	}

	@Override
	protected LocalTime getNow(Clock clock) {
		return LocalTime.now(clock);
	}

	@Override
	public LocalTimeConstraint<T> cast() {
		return this;
	}
}
